'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const debug = require('./debug.cjs');
const platform = require('./platform.cjs');
const style = require('./style.cjs');
const number = require('./number.cjs');

function callAllHandlers(...fns) {
  return function func(event) {
    fns.some((fn) => {
      fn?.(event);
      return event?.defaultPrevented;
    });
  };
}
function callAll(...fns) {
  return function mergedFn(arg) {
    fns.forEach((fn) => {
      fn?.(arg);
    });
  };
}

exports.DebugLevel = debug.DebugLevel;
exports.debug = debug.debug;
exports.setDebugLevel = debug.setDebugLevel;
exports.Platform = platform.Platform;
exports.closestBodyElement = platform.closestBodyElement;
exports.getOwnerDocument = platform.getOwnerDocument;
exports.getPlatform = platform.getPlatform;
exports.isSSR = platform.isSSR;
exports.setPlatform = platform.setPlatform;
exports.childSelectors = style.childSelectors;
exports.getAccent = style.getAccent;
exports.getAccentHover = style.getAccentHover;
exports.getAccentText = style.getAccentText;
exports.isDefaultAccent = style.isDefaultAccent;
exports.isPreferDarkMode = style.isPreferDarkMode;
exports.isPreferLightMode = style.isPreferLightMode;
exports.mediaQueryColorScheme = style.mediaQueryColorScheme;
exports.resolveThemeMode = style.resolveThemeMode;
exports.rgb = style.rgb;
exports.clampBigNumber = number.clampBigNumber;
exports.formatCurrency = number.formatCurrency;
exports.formatIntlNumber = number.formatIntlNumber;
exports.formatNumeric = number.formatNumeric;
exports.getCurrencyFormatter = number.getCurrencyFormatter;
exports.safelyFormatNumberWithFallback = number.safelyFormatNumberWithFallback;
exports.toNumber = number.toNumber;
exports.callAll = callAll;
exports.callAllHandlers = callAllHandlers;
