'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const vanilla = require('../node_modules/.pnpm/zustand@4.5.5_@types_react@18.3.7_immer@10.1.1_react@18.3.1/node_modules/zustand/esm/vanilla.cjs');
const middleware = require('../node_modules/.pnpm/zustand@4.5.5_@types_react@18.3.7_immer@10.1.1_react@18.3.1/node_modules/zustand/esm/middleware.cjs');
const immer = require('../node_modules/.pnpm/zustand@4.5.5_@types_react@18.3.7_immer@10.1.1_react@18.3.1/node_modules/zustand/esm/middleware/immer.cjs');
const immer$1 = require('immer');
const BigNumber = require('bignumber.js');
const style = require('../helpers/style.cjs');
const number = require('../helpers/number.cjs');
const themes_css = require('../styles/themes.css.cjs');
const override = require('../styles/override/override.cjs');

const STORAGE_NAME = "interchain-ui-store";
const store = vanilla.createStore(
  middleware.persist(
    immer.immer((set) => ({
      themeMode: null,
      theme: null,
      themeClass: "",
      themeClasses: [themes_css.lightThemeClass, themes_css.darkThemeClass],
      themeAccent: "blue",
      // Custom theme contract
      themeDefs: [],
      customTheme: null,
      overrideStyleManager: new override.OverrideStyleManager("light"),
      _hasHydrated: false,
      setTheme: (newTheme, themeClass) => set((state) => {
        state.theme = newTheme;
        state.themeClass = themeClass;
      }),
      setThemeMode: (newThemeMode) => set((state) => {
        const resolveSystemMode = (themeMode) => {
          if (themeMode === "system") {
            if (style.isPreferDarkMode()) {
              return ["dark", themes_css.darkThemeClass];
            }
            if (style.isPreferLightMode()) {
              return ["light", themes_css.lightThemeClass];
            }
          } else {
            const themeClass = {
              dark: themes_css.darkThemeClass,
              light: themes_css.lightThemeClass
            }[themeMode];
            return [themeMode, themeClass];
          }
        };
        const [resolvedTheme, resolvedClass] = resolveSystemMode(newThemeMode);
        state.overrideStyleManager?.update(null, resolvedTheme);
        state.themeMode = newThemeMode;
        state.theme = resolvedTheme;
        state.themeClass = resolvedClass;
      }),
      setThemeDefs: (defs) => {
        set((state) => {
          state.themeDefs = defs;
        });
      },
      setCustomTheme: (customTheme) => set((state) => {
        state.customTheme = customTheme;
        const currentState = immer$1.current(state);
        if (customTheme !== null) {
          const customThemeObj = currentState.themeDefs.find(
            (item) => item.name === customTheme
          );
          if (customThemeObj) {
            override.assignThemeVars(customThemeObj.vars, state.theme);
          }
        }
      }),
      setThemeAccent: (accent) => set((state) => {
        state.themeAccent = accent;
      }),
      formatNumber: (props) => {
        const formatter = number.getCurrencyFormatter("en-US", {
          currency: "USD",
          style: props.style
        });
        return number.safelyFormatNumberWithFallback(
          formatter,
          new BigNumber(props.value)
        );
      },
      setFormatNumberFn: (fn) => set((state) => {
        state.formatNumber = fn;
      }),
      setHasHydrated: (hasHydrated) => {
        set({
          _hasHydrated: hasHydrated
        });
      }
    })),
    {
      name: STORAGE_NAME,
      onRehydrateStorage: (state) => {
        return (persistedState) => {
          state.setHasHydrated(true);
          state.setThemeMode(persistedState.themeMode);
        };
      },
      // Only choose to persist theme preference, ignore other state
      partialize: (state) => ({
        themeMode: state.themeMode
      })
    }
  )
);

exports.STORAGE_NAME = STORAGE_NAME;
exports.store = store;
