'use strict';

Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: 'Module' } });

const vue = require('vue');
const clx = require('clsx');
const avatarName_vue_vue_type_script_setup_true_lang = require('../avatar-name/avatar-name.vue.cjs');
const avatar_css = require('../avatar/avatar.css.cjs');

const _hoisted_1 = ["src", "srcset", "alt", "width", "height", "referrerPolicy", "crossOrigin", "loading", "data-status"];
const _sfc_main = /* @__PURE__ */ vue.defineComponent({
  __name: "avatar-image",
  props: {
    src: {},
    srcSet: {},
    width: {},
    height: {},
    name: {},
    borderRadius: {},
    loading: {},
    ignoreFallback: { type: Boolean },
    referrerPolicy: {},
    crossOrigin: {},
    sizes: {},
    fallbackMode: {},
    onError: { type: Function },
    onLoad: { type: Function },
    getInitials: { type: Function },
    className: {},
    class: {},
    classList: {},
    children: {},
    forwardedRef: {}
  },
  setup(__props) {
    const props = __props;
    const status = vue.ref("pending");
    const transitionState = vue.ref("idle");
    const imgRef = vue.ref();
    const cleanupRef = vue.ref();
    vue.onUnmounted(() => {
      if (typeof cleanupRef.value === "function") {
        cleanupRef.value();
      }
    });
    vue.watch(
      () => [props.src],
      () => {
        status.value = props.src ? "loading" : "pending";
      },
      { immediate: true }
    );
    vue.watch(
      () => [
        status.value,
        props.ignoreFallback,
        props.crossOrigin,
        props.srcSet,
        props.sizes,
        props.onLoad,
        props.onError,
        props.loading
      ],
      () => {
        if (props.ignoreFallback) return;
        if (status.value === "loading") {
          load();
        }
        cleanupRef.value = () => {
          flush();
        };
      },
      { immediate: true }
    );
    function load() {
      if (!props.src) return;
      flush();
      const img = new Image();
      img.src = props.src;
      if (props.crossOrigin) img.crossOrigin = props.crossOrigin;
      if (props.srcSet) img.srcset = props.srcSet;
      if (props.sizes) img.sizes = props.sizes;
      if (props.loading) img.loading = props.loading;
      img.onload = (event) => {
        flush();
        status.value = "loaded";
        transitionState.value = "entered";
        props.onLoad?.(event);
      };
      img.onerror = (error) => {
        flush();
        status.value = "failed";
        props.onError?.(error);
      };
      imgRef.value = img;
    }
    function flush() {
      if (imgRef.value) {
        imgRef.value.onload = null;
        imgRef.value.onerror = null;
        imgRef.value = null;
        transitionState.value = "idle";
      }
    }
    function getResolvedStatus() {
      return props.ignoreFallback ? "loaded" : status.value;
    }
    function getShowFallback() {
      return !props.src || getResolvedStatus() !== "loaded";
    }
    function getInlineStyles() {
      return {
        opacity: `${transitionState.value === "idle" ? 0 : 1}`,
        transition: transitionState.value === "entered" ? "opacity 150ms cubic-bezier(0.7, 0, 0.84, 0)" : "none !important"
      };
    }
    return (_ctx, _cache) => {
      return getShowFallback() ? (vue.openBlock(), vue.createBlock(vue.unref(avatarName_vue_vue_type_script_setup_true_lang.default), {
        key: 0,
        name: _ctx.name,
        getInitials: _ctx.getInitials,
        showInitials: _ctx.fallbackMode === "initials"
      }, null, 8, ["name", "getInitials", "showInitials"])) : (vue.openBlock(), vue.createElementBlock("img", {
        key: 1,
        ref_key: "imgRef",
        ref: imgRef,
        src: _ctx.src,
        srcset: _ctx.srcSet,
        alt: _ctx.name,
        onLoad: _cache[0] || (_cache[0] = //@ts-ignore
        (...args) => _ctx.onLoad && _ctx.onLoad(...args)),
        width: _ctx.width,
        height: _ctx.height,
        referrerPolicy: _ctx.referrerPolicy,
        crossOrigin: _ctx.crossOrigin ?? void 0,
        loading: _ctx.loading,
        "data-status": getResolvedStatus(),
        style: vue.normalizeStyle(getInlineStyles()),
        class: vue.normalizeClass(vue.unref(clx)(vue.unref(avatar_css.avatarImg), _ctx.className))
      }, null, 46, _hoisted_1));
    };
  }
});

exports.default = _sfc_main;
