import { defineComponent, ref, onMounted, onUnmounted, openBlock, createElementBlock, normalizeStyle, unref, normalizeClass, createVNode, withCtx, createTextVNode, toDisplayString, createBlock } from 'vue';
import clx from 'clsx';
import copy from 'copy-to-clipboard';
import _sfc_main$2 from '../icon/icon.vue.js';
import { containerStyle, iconStyle, textStyle, truncateEndStyle } from './clipboard-copy-text.css.js';
import { store } from '../../models/store.js';
import { truncateTextMiddle } from '../../helpers/string.js';
import { clipboardCopyTextOverrides } from './clipboard-copy-text.helper.js';
import _sfc_main$1 from '../text/text.vue.js';

const _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "clipboard-copy-text",
  props: {
    text: {},
    truncate: {},
    midTruncateLimit: {},
    onCopied: { type: Function },
    className: {},
    class: {},
    classList: {},
    children: {},
    forwardedRef: {}
  },
  setup(__props) {
    const props = __props;
    const idle = ref(true);
    const internalTheme = ref("light");
    const overrideManager = ref(null);
    const cleanupRef = ref();
    onMounted(() => {
      internalTheme.value = store.getState().theme;
      overrideManager.value = store.getState().overrideStyleManager;
      cleanupRef.value = store.subscribe((newState) => {
        internalTheme.value = newState.theme;
        overrideManager.value = newState.overrideStyleManager;
      });
    });
    onUnmounted(() => {
      if (typeof cleanupRef.value === "function") cleanupRef.value();
    });
    function transform(text) {
      if (props.truncate === "middle") {
        const truncateLength = {
          lg: 14,
          md: 16,
          sm: 18
        };
        return truncateTextMiddle(
          text,
          truncateLength[props.midTruncateLimit ?? "md"]
        );
      }
      return text;
    }
    function handleOnClick(event) {
      const success = copy(props.text);
      if (success) {
        props.onCopied?.(event);
        idle.value = false;
        setTimeout(() => {
          idle.value = true;
        }, 1e3);
      }
    }
    function getTruncateClass() {
      return clx(textStyle, props.truncate === "end" && truncateEndStyle);
    }
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        onClick: _cache[0] || (_cache[0] = ($event) => handleOnClick($event)),
        style: normalizeStyle(overrideManager.value?.applyOverrides(unref(clipboardCopyTextOverrides).name)),
        class: normalizeClass(unref(clx)(unref(containerStyle)[internalTheme.value], _ctx.className))
      }, [
        createVNode(unref(_sfc_main$1), {
          color: "$textSecondary",
          class: normalizeClass(getTruncateClass())
        }, {
          default: withCtx(() => [
            createTextVNode(toDisplayString(transform(_ctx.text)), 1)
          ]),
          _: 1
        }, 8, ["class"]),
        idle.value ? (openBlock(), createBlock(unref(_sfc_main$2), {
          key: 0,
          name: "copy",
          size: "$md",
          class: normalizeClass(unref(iconStyle).idle)
        }, null, 8, ["class"])) : (openBlock(), createBlock(unref(_sfc_main$2), {
          key: 1,
          name: "checkboxCircle",
          size: "$md",
          class: normalizeClass(unref(iconStyle).copied[internalTheme.value])
        }, null, 8, ["class"]))
      ], 6);
    };
  }
});

export { _sfc_main as default };
