import { defineComponent, ref, onMounted, onUnmounted, watch, openBlock, createElementBlock, normalizeClass, unref, createElementVNode, createBlock, withCtx, createVNode, Fragment, renderList, createCommentVNode } from 'vue';
import anime from 'animejs';
import clx from 'clsx';
import _sfc_main$2 from '../box/box.vue.js';
import _sfc_main$1 from '../fade-in/fade-in.vue.js';
import _sfc_main$3 from '../connect-modal-wallet-button/connect-modal-wallet-button.vue.js';
import { container, walletList, squareWallets, listWallets } from './connect-modal-wallet-list.css.js';
import { bottomShadow } from '../shared/shared.css.js';
import { store } from '../../models/store.js';

const _hoisted_1 = ["data-has-list-wallets"];
const _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "connect-modal-wallet-list",
  props: {
    wallets: {},
    onWalletItemClick: { type: Function },
    className: {},
    class: {},
    classList: {},
    children: {},
    forwardedRef: {}
  },
  setup(__props) {
    const props = __props;
    const displayBlur = ref(false);
    const internalTheme = ref("light");
    const measureRef = ref();
    const shadowRef = ref();
    const animationRef = ref();
    const cleanupRef = ref();
    onMounted(() => {
      internalTheme.value = store.getState().theme;
      const unsubTheme = store.subscribe((newState) => {
        internalTheme.value = newState.theme;
      });
      if (measureRef.value) {
        if (measureRef.value.clientHeight >= 320) {
          displayBlur.value = true;
        } else {
          displayBlur.value = false;
        }
        const scrollHandler = () => {
          const height = Math.abs(
            measureRef.value.scrollHeight - measureRef.value.clientHeight - measureRef.value.scrollTop
          );
          if (height < 1) {
            displayBlur.value = false;
          } else {
            displayBlur.value = true;
          }
        };
        measureRef.value.addEventListener("scroll", scrollHandler);
        cleanupRef.value = () => {
          unsubTheme();
          if (measureRef.value) {
            measureRef.value.removeEventListener("scroll", scrollHandler);
          }
        };
      }
    });
    onUnmounted(() => {
      if (typeof cleanupRef.value === "function") cleanupRef.value();
    });
    watch(
      () => [displayBlur.value, shadowRef.value],
      () => {
        if (!shadowRef.value) return;
        if (shadowRef.value && !animationRef.value) {
          animationRef.value = anime({
            targets: shadowRef.value,
            opacity: [0, 1],
            height: [0, 36],
            delay: 50,
            duration: 250,
            direction: `alternate`,
            loop: false,
            autoplay: false,
            easing: `easeInOutSine`
          });
        }
        if (displayBlur.value) {
          animationRef.value?.restart();
        } else {
          animationRef.value?.reverse();
        }
      },
      { immediate: true }
    );
    function onWalletItemClickAsync(exec) {
      void async function() {
        await exec();
      }();
    }
    function getListShapeWallets() {
      return props.wallets.filter((w) => w.shape === "list");
    }
    function showSquareShapeWallets() {
      return props.wallets.slice(0, 2).every((wallet) => wallet.shape === "square");
    }
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        class: normalizeClass(unref(clx)(unref(container), _ctx.className))
      }, [
        createElementVNode("div", {
          ref_key: "measureRef",
          ref: measureRef,
          "data-has-list-wallets": getListShapeWallets().length > 0,
          class: normalizeClass(unref(walletList))
        }, [
          showSquareShapeWallets() ? (openBlock(), createBlock(unref(_sfc_main$1), {
            key: 0,
            isVisible: showSquareShapeWallets()
          }, {
            default: withCtx(() => [
              createVNode(unref(_sfc_main$2), {
                display: "grid",
                class: normalizeClass(unref(squareWallets))
              }, {
                default: withCtx(() => [
                  (openBlock(true), createElementBlock(Fragment, null, renderList(_ctx.wallets.slice(0, 2), (wallet, index) => {
                    return openBlock(), createBlock(unref(_sfc_main$3), {
                      key: `${wallet.name}-${index}`,
                      variant: "square",
                      name: wallet.prettyName ?? wallet.name,
                      logo: wallet.logo,
                      subLogo: wallet.subLogo,
                      btmLogo: wallet.btmLogo,
                      onClick: (event) => onWalletItemClickAsync(
                        async () => _ctx.onWalletItemClick?.(wallet.originalWallet)
                      )
                    }, null, 8, ["name", "logo", "subLogo", "btmLogo", "onClick"]);
                  }), 128))
                ]),
                _: 1
              }, 8, ["class"])
            ]),
            _: 1
          }, 8, ["isVisible"])) : createCommentVNode("", true),
          getListShapeWallets().length > 0 ? (openBlock(), createBlock(unref(_sfc_main$1), {
            key: 1,
            isVisible: getListShapeWallets().length > 0
          }, {
            default: withCtx(() => [
              createVNode(unref(_sfc_main$2), {
                display: "grid",
                class: normalizeClass(unref(listWallets))
              }, {
                default: withCtx(() => [
                  (openBlock(true), createElementBlock(Fragment, null, renderList(getListShapeWallets(), (wallet, index) => {
                    return openBlock(), createBlock(unref(_sfc_main$3), {
                      key: `${wallet.name}-${index}`,
                      variant: "list",
                      name: wallet.prettyName ?? wallet.name,
                      logo: wallet.logo,
                      badge: wallet.badge,
                      subLogo: wallet.subLogo,
                      btmLogo: wallet.btmLogo,
                      onClick: (event) => onWalletItemClickAsync(
                        async () => _ctx.onWalletItemClick?.(wallet.originalWallet)
                      )
                    }, null, 8, ["name", "logo", "badge", "subLogo", "btmLogo", "onClick"]);
                  }), 128))
                ]),
                _: 1
              }, 8, ["class"])
            ]),
            _: 1
          }, 8, ["isVisible"])) : createCommentVNode("", true)
        ], 10, _hoisted_1),
        createElementVNode("div", {
          ref_key: "shadowRef",
          ref: shadowRef,
          class: normalizeClass(unref(bottomShadow)[internalTheme.value])
        }, null, 2)
      ], 2);
    };
  }
});

export { _sfc_main as default };
