'use strict';

Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: 'Module' } });

const vue = require('vue');
const anime = require('animejs');

const _sfc_main = /* @__PURE__ */ vue.defineComponent({
  __name: "fade-in",
  props: {
    isVisible: { type: Boolean },
    durationMs: {},
    delayMs: {},
    children: {}
  },
  setup(__props) {
    const props = __props;
    const fadeInAnimationRef = vue.ref();
    const fadeOutAnimationRef = vue.ref();
    const elementRef = vue.ref();
    vue.watch(
      () => [props.delayMs, props.durationMs, props.isVisible],
      () => {
        const delaySetting = props.delayMs || 100;
        const durationSetting = props.durationMs || 250;
        if (elementRef.value && !fadeInAnimationRef.value && !fadeOutAnimationRef.value) {
          fadeInAnimationRef.value = anime({
            targets: elementRef.value,
            opacity: [0, 1],
            delay: delaySetting,
            duration: durationSetting,
            direction: `alternate`,
            loop: false,
            autoplay: false,
            easing: "spring(1, 80, 10, 0)"
          });
          fadeOutAnimationRef.value = anime({
            targets: elementRef.value,
            opacity: [1, 0],
            delay: delaySetting,
            duration: durationSetting,
            direction: `alternate`,
            loop: false,
            autoplay: false,
            easing: "spring(1, 80, 10, 0)"
          });
        }
        if (props.isVisible) {
          fadeInAnimationRef.value?.restart();
        } else {
          fadeOutAnimationRef.value?.restart();
        }
      },
      { immediate: true }
    );
    return (_ctx, _cache) => {
      return vue.openBlock(), vue.createElementBlock("div", {
        ref_key: "elementRef",
        ref: elementRef
      }, [
        vue.renderSlot(_ctx.$slots, "default")
      ], 512);
    };
  }
});

exports.default = _sfc_main;
