import { defineComponent, ref, watch, openBlock, createElementBlock, renderSlot } from 'vue';
import anime from 'animejs';

const _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "fade-in",
  props: {
    isVisible: { type: Boolean },
    durationMs: {},
    delayMs: {},
    children: {}
  },
  setup(__props) {
    const props = __props;
    const fadeInAnimationRef = ref();
    const fadeOutAnimationRef = ref();
    const elementRef = ref();
    watch(
      () => [props.delayMs, props.durationMs, props.isVisible],
      () => {
        const delaySetting = props.delayMs || 100;
        const durationSetting = props.durationMs || 250;
        if (elementRef.value && !fadeInAnimationRef.value && !fadeOutAnimationRef.value) {
          fadeInAnimationRef.value = anime({
            targets: elementRef.value,
            opacity: [0, 1],
            delay: delaySetting,
            duration: durationSetting,
            direction: `alternate`,
            loop: false,
            autoplay: false,
            easing: "spring(1, 80, 10, 0)"
          });
          fadeOutAnimationRef.value = anime({
            targets: elementRef.value,
            opacity: [1, 0],
            delay: delaySetting,
            duration: durationSetting,
            direction: `alternate`,
            loop: false,
            autoplay: false,
            easing: "spring(1, 80, 10, 0)"
          });
        }
        if (props.isVisible) {
          fadeInAnimationRef.value?.restart();
        } else {
          fadeOutAnimationRef.value?.restart();
        }
      },
      { immediate: true }
    );
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        ref_key: "elementRef",
        ref: elementRef
      }, [
        renderSlot(_ctx.$slots, "default")
      ], 512);
    };
  }
});

export { _sfc_main as default };
