import { defineComponent, ref, onMounted, watch, openBlock, createElementBlock, normalizeClass, toDisplayString, createCommentVNode, createElementVNode } from 'vue';
import qrcodegen from './qrcodegen/qrcodegen.js';
import { DEFAULT_SIZE, DEFAULT_LEVEL, DEFAULT_BGCOLOR, DEFAULT_FGCOLOR, DEFAULT_INCLUDEMARGIN, ERROR_LEVEL_MAP } from './qrcode.types.js';
import { excavateModules, getMarginSize, getImageSettings, generatePath } from './qrcode.helpers.js';

const _hoisted_1 = ["height", "width", "viewBox"];
const _hoisted_2 = { key: 0 };
const _hoisted_3 = ["fill", "d"];
const _hoisted_4 = ["fill", "d"];
const _hoisted_5 = ["href", "height", "width", "x", "y"];
const _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "qrcode",
  props: {
    value: { default: void 0 },
    size: { default: DEFAULT_SIZE },
    level: { default: DEFAULT_LEVEL },
    bgColor: { default: DEFAULT_BGCOLOR },
    fgColor: { default: DEFAULT_FGCOLOR },
    includeMargin: { type: Boolean, default: DEFAULT_INCLUDEMARGIN },
    marginSize: { default: void 0 },
    imageSettings: { default: void 0 },
    title: { default: void 0 },
    className: { default: void 0 },
    class: {},
    classList: {},
    children: {},
    forwardedRef: {}
  },
  setup(__props) {
    const props = __props;
    const available = ref(false);
    const cells = ref(null);
    const calculatedImageSettings = ref(null);
    const margin = ref(null);
    const fgPath = ref(null);
    onMounted(() => {
      cells.value = genCells();
    });
    watch(
      () => [props.imageSettings, calculatedImageSettings.value],
      () => {
        if (props.imageSettings != null && calculatedImageSettings.value != null) {
          if (calculatedImageSettings.value.excavation != null) {
            cells.value = excavateModules(
              cells.value,
              calculatedImageSettings.value.excavation
            );
          }
          if (!available.value) {
            available.value = true;
          }
        }
      },
      { immediate: true }
    );
    watch(
      () => [
        props.size,
        props.imageSettings,
        props.includeMargin,
        props.marginSize,
        cells.value
      ],
      () => {
        const newMargin = getMarginSize(props.includeMargin, props.marginSize);
        margin.value = newMargin;
        calculatedImageSettings.value = getImageSettings(
          cells.value,
          props.size,
          newMargin,
          props.imageSettings
        );
        fgPath.value = generateNewPath(newMargin);
      },
      { immediate: true }
    );
    function generateNewPath(newMargin) {
      return generatePath(cells.value ?? [], newMargin);
    }
    function genCells() {
      return qrcodegen.QrCode.encodeText(
        props.value,
        ERROR_LEVEL_MAP[props.level]
      ).getModules();
    }
    function numCells() {
      return (cells.value?.length ?? 0) + (margin.value ?? 0) * 2;
    }
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("svg", {
        height: _ctx.size,
        width: _ctx.size,
        viewBox: `0 0 ${numCells()} ${numCells()}`,
        class: normalizeClass(props.class ?? props.className)
      }, [
        !!_ctx.title ? (openBlock(), createElementBlock("title", _hoisted_2, toDisplayString(_ctx.title), 1)) : createCommentVNode("", true),
        createElementVNode("path", {
          "shape-rendering": "crispEdges",
          fill: _ctx.bgColor,
          d: `M0,0 h${numCells()}v${numCells()}H0z`
        }, null, 8, _hoisted_3),
        createElementVNode("path", {
          "shape-rendering": "crispEdges",
          fill: _ctx.fgColor,
          d: fgPath.value
        }, null, 8, _hoisted_4),
        available.value && !!_ctx.imageSettings?.src ? (openBlock(), createElementBlock("image", {
          key: 1,
          preserveAspectRatio: "none",
          href: _ctx.imageSettings.src,
          height: calculatedImageSettings.value.h,
          width: calculatedImageSettings.value.w,
          x: calculatedImageSettings.value.x + margin.value,
          y: calculatedImageSettings.value.y + margin.value
        }, null, 8, _hoisted_5)) : createCommentVNode("", true)
      ], 10, _hoisted_1);
    };
  }
});

export { _sfc_main as default };
