'use strict';

Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: 'Module' } });

const vue = require('vue');
const clx = require('clsx');
const anime = require('animejs');
const lodash = require('lodash');
const stack_vue_vue_type_script_setup_true_lang = require('../stack/stack.vue.cjs');
const box_vue_vue_type_script_setup_true_lang = require('../box/box.vue.cjs');
const text_vue_vue_type_script_setup_true_lang = require('../text/text.vue.cjs');
const icon_vue_vue_type_script_setup_true_lang = require('../icon/icon.vue.cjs');
const store = require('../../models/store.cjs');
const reveal_css = require('./reveal.css.cjs');

const _sfc_main = /* @__PURE__ */ vue.defineComponent({
  __name: "reveal",
  props: {
    hideThresholdHeight: { default: 500 },
    showMoreLabel: { default: "Show more" },
    showLessLabel: { default: "Show less" },
    fadeBackground: {},
    children: { default: void 0 },
    className: { default: void 0 },
    class: {},
    classList: {},
    forwardedRef: {}
  },
  setup(__props) {
    const props = __props;
    const internalTheme = vue.ref("light");
    const isVisible = vue.ref(false);
    const eleHeight = vue.ref();
    const isVisibleRef = vue.ref();
    const animationRef = vue.ref();
    const elementRef = vue.ref();
    const cleanupRef = vue.ref();
    const resizeListenerRef = vue.ref();
    vue.onMounted(() => {
      internalTheme.value = store.store.getState().theme;
      cleanupRef.value = store.store.subscribe((newState, prevState) => {
        internalTheme.value = newState.theme;
      });
      setTimeout(() => {
        if (!elementRef.value) return;
        const TOGGLE_SPACE = 40;
        if (elementRef.value.offsetHeight > props.hideThresholdHeight) {
          resizeListenerRef.value = lodash.debounce(() => {
            if (!elementRef.value) return;
            elementRef.value.style.height = "auto";
            eleHeight.value = elementRef.value.offsetHeight + TOGGLE_SPACE;
            elementRef.value.style.height = isVisibleRef.value ? `${eleHeight.value}px` : `${props.hideThresholdHeight}px`;
            updateAnimationRef();
          }, 100);
          window.addEventListener("resize", resizeListenerRef.value);
          setTimeout(() => {
            if (!eleHeight.value) {
              eleHeight.value = elementRef.value.offsetHeight + TOGGLE_SPACE;
            }
            updateAnimationRef();
          }, 100);
        }
      }, 100);
    });
    vue.onUnmounted(() => {
      if (typeof cleanupRef.value === "function") cleanupRef.value();
      if (window) {
        window.removeEventListener("resize", resizeListenerRef.value);
      }
    });
    function toggle() {
      isVisibleRef.value = !isVisible.value;
      isVisible.value = !isVisible.value;
    }
    function updateAnimationRef() {
      animationRef.value = anime({
        targets: elementRef.value,
        height: [props.hideThresholdHeight, eleHeight.value],
        duration: 250,
        direction: `alternate`,
        loop: false,
        autoplay: false,
        easing: `easeInOutSine`
      });
      isVisible.value = false;
    }
    return (_ctx, _cache) => {
      return vue.openBlock(), vue.createElementBlock("div", {
        ref_key: "elementRef",
        ref: elementRef,
        class: vue.normalizeClass(vue.unref(clx)(reveal_css.container, _ctx.className))
      }, [
        vue.renderSlot(_ctx.$slots, "default"),
        vue.createVNode(vue.unref(stack_vue_vue_type_script_setup_true_lang.default), {
          direction: "vertical",
          attributes: {
            display: !isVisible.value ? "flex" : "none",
            cursor: "pointer",
            position: "absolute",
            bottom: 0,
            left: 0,
            right: 0,
            width: "$full",
            justifyContent: "center",
            alignItems: "flex-end",
            height: "$27"
          }
        }, {
          default: vue.withCtx(() => [
            vue.createVNode(vue.unref(box_vue_vue_type_script_setup_true_lang.default), {
              height: "$23",
              flexShrink: "0",
              width: "$full",
              class: vue.normalizeClass(reveal_css.shadow[internalTheme.value])
            }, null, 8, ["class"]),
            vue.createVNode(vue.unref(box_vue_vue_type_script_setup_true_lang.default), {
              width: "$full",
              display: "flex",
              alignItems: "center",
              justifyContent: "center",
              flex: "1",
              backgroundColor: internalTheme.value === "light" ? "$white" : "$gray900",
              attributes: {
                onClick: () => {
                  toggle();
                  animationRef.value?.play();
                },
                "data-part-id": "reveal-showmore"
              }
            }, {
              default: vue.withCtx(() => [
                vue.createVNode(vue.unref(text_vue_vue_type_script_setup_true_lang.default), {
                  color: "$textSecondary",
                  fontWeight: "$semibold",
                  attributes: {
                    marginRight: "$5"
                  }
                }, {
                  default: vue.withCtx(() => [
                    vue.createTextVNode(vue.toDisplayString(_ctx.showMoreLabel), 1)
                  ]),
                  _: 1
                }),
                vue.createVNode(vue.unref(icon_vue_vue_type_script_setup_true_lang.default), {
                  name: "arrowDownS",
                  color: "$textSecondary"
                })
              ]),
              _: 1
            }, 8, ["backgroundColor", "attributes"])
          ]),
          _: 1
        }, 8, ["attributes"]),
        vue.createVNode(vue.unref(stack_vue_vue_type_script_setup_true_lang.default), {
          attributes: {
            display: !!isVisible.value ? "flex" : "none",
            cursor: "pointer",
            position: "absolute",
            bottom: 0,
            width: "100%",
            justifyContent: "center",
            alignItems: "flex-end",
            height: "fit-content"
          }
        }, {
          default: vue.withCtx(() => [
            vue.createVNode(vue.unref(box_vue_vue_type_script_setup_true_lang.default), {
              display: "flex",
              alignItems: "center",
              attributes: {
                onClick: () => {
                  toggle();
                  animationRef.value?.reverse();
                  animationRef.value?.play();
                },
                "data-part-id": "reveal-showless"
              }
            }, {
              default: vue.withCtx(() => [
                vue.createVNode(vue.unref(text_vue_vue_type_script_setup_true_lang.default), {
                  color: "$textSecondary",
                  fontWeight: "$semibold",
                  attributes: {
                    marginRight: "$5"
                  }
                }, {
                  default: vue.withCtx(() => [
                    vue.createTextVNode(vue.toDisplayString(_ctx.showLessLabel), 1)
                  ]),
                  _: 1
                }),
                vue.createVNode(vue.unref(icon_vue_vue_type_script_setup_true_lang.default), {
                  name: "arrowUpS",
                  color: "$textSecondary"
                })
              ]),
              _: 1
            }, 8, ["attributes"])
          ]),
          _: 1
        }, 8, ["attributes"])
      ], 2);
    };
  }
});

exports.default = _sfc_main;
