import { defineComponent, ref, onMounted, onUnmounted, computed, onUpdated, watch, openBlock, createElementBlock, unref, normalizeStyle, normalizeClass, createCommentVNode, createVNode, Fragment, createTextVNode, toDisplayString, createBlock, createElementVNode } from 'vue';
import { assignInlineVars } from '@vanilla-extract/dynamic';
import clx from 'clsx';
import { store } from '../../models/store.js';
import _sfc_main$1 from '../icon/icon.vue.js';
import { TOAST_LIFETIME, GAP, TIME_BEFORE_UNMOUNT, SWIPE_TRESHOLD } from './toast.constants.js';
import { toast, indexVar, toastsBeforeVar, zIndexVar, offsetVar, initialHeightVar, swipeAmountVar, toastCloseButton, toastIcon, toastSpinner, toastContent, toastTitle, toastDescription, toastCancelButton, toastButton } from './toast.css.js';

const _hoisted_1 = ["aria-live", "data-theme", "data-mounted", "data-promise", "data-removed", "data-visible", "data-y-position", "data-x-position", "data-index", "data-front", "data-swiping", "data-type", "data-invert", "data-swipe-out", "data-expanded"];
const _hoisted_2 = ["data-disabled"];
const _hoisted_3 = /* @__PURE__ */ createElementVNode("svg", {
  xmlns: "http://www.w3.org/2000/svg",
  width: "12",
  height: "12",
  viewBox: "0 0 24 24",
  fill: "none",
  stroke: "currentColor",
  "stroke-width": "1.5",
  "stroke-line-cap": "round",
  "stroke-line-join": "round"
}, [
  /* @__PURE__ */ createElementVNode("line", {
    x1: "18",
    y1: "6",
    x2: "6",
    y2: "18"
  }),
  /* @__PURE__ */ createElementVNode("line", {
    x1: "6",
    y1: "6",
    x2: "18",
    y2: "18"
  })
], -1);
const _hoisted_4 = [
  _hoisted_3
];
const _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "toast",
  props: {
    toast: {},
    toasts: {},
    index: {},
    expanded: { type: Boolean },
    invert: { type: Boolean },
    heights: {},
    setHeights: { type: Function },
    removeToast: { type: Function },
    position: {},
    visibleToasts: {},
    expandByDefault: { type: Boolean },
    closeButton: { type: Boolean },
    interacting: { type: Boolean },
    rawCSS: {},
    duration: {},
    className: {},
    descriptionClassName: {},
    class: {},
    classList: {},
    children: {},
    forwardedRef: {}
  },
  setup(__props) {
    const props = __props;
    const theme = ref("light");
    const mounted = ref(false);
    const removed = ref(false);
    const swiping = ref(false);
    const swipeOut = ref(false);
    const offsetBeforeRemove = ref(0);
    const initialHeight = ref(0);
    const swipeAmount = ref("0px");
    const toastRef = ref();
    const cleanupTimerRef = ref();
    const restoreHeightsRef = ref();
    const closeTimerStartTimeRef = ref();
    const offset = ref();
    const lastCloseTimerStartTimeRef = ref();
    const pointerStartRef = ref();
    const closeTimerRemainingTimeRef = ref();
    const cleanupUIStore = ref();
    onMounted(() => {
      mounted.value = true;
      theme.value = store.getState().theme.value;
      cleanupUIStore.value = store.subscribe((newState) => {
        theme.value = newState.theme.value;
      });
    });
    onUnmounted(() => {
      if (typeof cleanupTimerRef.value === "function") {
        cleanupTimerRef.value();
      }
      if (typeof restoreHeightsRef.value === "function") {
        restoreHeightsRef.value();
      }
      if (typeof cleanupUIStore.value === "function") {
        cleanupUIStore.value();
      }
    });
    const isFront = computed(() => {
      return props.index === 0;
    });
    const isVisible = computed(() => {
      return props.index + 1 <= props.visibleToasts;
    });
    const toastType = computed(() => {
      return props.toast.type;
    });
    const positionTuple = computed(() => {
      const [y, x] = props.position.split("-");
      return {
        x,
        y
      };
    });
    const invert = computed(() => {
      return props.toast.invert || props.invert;
    });
    const disabled = computed(() => {
      return props.toast.type === "loading";
    });
    const statusIconName = computed(() => {
      if (props.toast.type === "success") return "checkFill";
      if (props.toast.type === "error") return "errorWarningFill";
      return null;
    });
    const heightIndex = computed(() => {
      return props.heights.findIndex(
        (height) => height.toastId === props.toast.id
      ) || 0;
    });
    computed(() => {
      return props.toast.duration || props.duration || TOAST_LIFETIME;
    });
    const toastsHeightBefore = computed(() => {
      return props.heights.reduce((prev, curr, reducerIndex) => {
        if (reducerIndex >= heightIndex) {
          return prev;
        }
        return prev + curr.height;
      }, 0);
    });
    onUpdated(() => {
      offset.value = heightIndex * GAP + toastsHeightBefore;
    });
    watch(
      () => [
        mounted.value,
        props.toast.title,
        props.toast.description,
        props.toast.id
      ],
      () => {
        if (!mounted.value) {
          return;
        }
        const toastNode = toastRef.value;
        const originalHeight = toastNode.style.height;
        toastNode.style.height = "auto";
        const newHeight = toastNode.getBoundingClientRect().height;
        toastNode.style.height = originalHeight;
        initialHeight.value = newHeight;
        const calcNewHeights = () => {
          const oldHeights = props.heights;
          const alreadyExists = oldHeights.find(
            (height) => height.toastId === props.toast.id
          );
          if (!alreadyExists) {
            return [
              {
                toastId: props.toast.id,
                height: newHeight
              },
              ...oldHeights
            ];
          } else {
            return oldHeights.map(
              (height) => height.toastId === props.toast.id ? {
                ...height,
                height: newHeight
              } : height
            );
          }
        };
        props.setHeights(calcNewHeights());
      },
      { immediate: true }
    );
    watch(
      () => [
        props.expanded,
        props.interacting,
        props.expandByDefault,
        props.toast,
        props.duration,
        props.toast.promise,
        deleteToast,
        toastType
      ],
      () => {
        if (typeof cleanupTimerRef.value === "function") {
          cleanupTimerRef.value();
        }
        if (props.toast.promise && toastType === "loading" || props.toast.duration === Infinity) {
          return;
        }
        let timeoutId;
        const pauseTimer = () => {
          if (lastCloseTimerStartTimeRef.value < closeTimerStartTimeRef.value) {
            const elapsedTime = (/* @__PURE__ */ new Date()).getTime() - closeTimerStartTimeRef.value;
            closeTimerRemainingTimeRef.value = closeTimerRemainingTimeRef.value - elapsedTime;
          }
          lastCloseTimerStartTimeRef.value = (/* @__PURE__ */ new Date()).getTime();
        };
        const startTimer = () => {
          closeTimerStartTimeRef.value = (/* @__PURE__ */ new Date()).getTime();
          timeoutId = setTimeout(() => {
            props.toast.onAutoClose?.(props.toast);
            deleteToast();
          }, closeTimerRemainingTimeRef.value);
        };
        if (props.expanded || props.interacting) {
          pauseTimer();
        } else {
          startTimer();
        }
        cleanupTimerRef.value = () => clearTimeout(timeoutId);
      },
      { immediate: true }
    );
    watch(
      () => [props.toast.id],
      () => {
        const toastNode = toastRef.value;
        if (toastNode) {
          const height = toastNode.getBoundingClientRect().height;
          initialHeight.value = height;
          const oldHeights = props.heights;
          props.setHeights([
            {
              toastId: props.toast.id,
              height
            },
            ...oldHeights
          ]);
          restoreHeightsRef.value = () => {
            const oldHeights2 = props.heights;
            props.setHeights(
              oldHeights2.filter((height2) => height2.toastId !== props.toast.id)
            );
          };
        }
      },
      { immediate: true }
    );
    watch(
      () => [props.toast.delete],
      () => {
        if (props.toast.delete) {
          deleteToast();
        }
      },
      { immediate: true }
    );
    function pxToNum(cssValue) {
      return Number(cssValue.replace("px", "")) || 0;
    }
    function deleteToast() {
      removed.value = true;
      offsetBeforeRemove.value = offset.value;
      const oldHeights = props.heights;
      props.setHeights(
        oldHeights.filter((height) => height.toastId !== props.toast.id)
      );
      setTimeout(() => {
        props.removeToast(props.toast);
      }, TIME_BEFORE_UNMOUNT);
    }
    function handleCloseButtonClick() {
      if (disabled) {
        return;
      }
      deleteToast();
      props.toast.onDismiss?.(props.toast);
    }
    function handleCancelButtonClick() {
      deleteToast();
      if (props.toast.cancel?.onClick) {
        props.toast.cancel.onClick();
      }
    }
    function handleActionButtonClick(event) {
      props.toast.action?.onClick(event);
      if (event.defaultPrevented) {
        return;
      }
      deleteToast();
    }
    function handleOnPointerDown(event) {
      if (disabled) return;
      offsetBeforeRemove.value = offset.value;
      event.target.setPointerCapture(event.pointerId);
      if (event.target.tagName === "BUTTON") return;
      swiping.value = true;
      pointerStartRef.value = {
        x: event.clientX,
        y: event.clientY
      };
    }
    function handleOnPointerUp() {
      if (swipeOut.value) {
        return;
      }
      pointerStartRef.value = null;
      const swipeAmountNum = Number(
        pxToNum(toastRef.value?.style.getPropertyValue(`${swipeAmountVar}`))
      );
      if (Math.abs(swipeAmountNum) >= SWIPE_TRESHOLD) {
        offsetBeforeRemove.value = offset.value;
        props.toast.onDismiss?.(props.toast);
        deleteToast();
        swipeOut.value = true;
        return;
      }
      swipeAmount.value = "0px";
      swiping.value = false;
    }
    function handleOnPointerMove(event) {
      if (!pointerStartRef.value) return;
      const yPosition = event.clientY - pointerStartRef.value.y;
      const xPosition = event.clientX - pointerStartRef.value.x;
      const clamp = positionTuple.y === "top" ? Math.min : Math.max;
      const clampedY = clamp(0, yPosition);
      const swipeStartThreshold = event.pointerType === "touch" ? 10 : 2;
      const isAllowedToSwipe = Math.abs(clampedY) > swipeStartThreshold;
      if (isAllowedToSwipe) {
        swipeAmount.value = `${yPosition}px`;
      } else if (Math.abs(xPosition) > swipeStartThreshold) {
        pointerStartRef.value = null;
      }
    }
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("li", {
        "aria-atomic": "true",
        role: "status",
        "data-interchain-toast": "",
        "aria-live": unref(toast).important ? "assertive" : "polite",
        tabIndex: 0,
        ref_key: "toastRef",
        ref: toastRef,
        "data-styled": true,
        "data-theme": theme.value,
        "data-mounted": mounted.value,
        "data-promise": Boolean(unref(toast).promise),
        "data-removed": removed.value,
        "data-visible": isVisible.value,
        "data-y-position": positionTuple.value.y,
        "data-x-position": positionTuple.value.x,
        "data-index": _ctx.index,
        "data-front": isFront.value,
        "data-swiping": swiping.value,
        "data-type": toastType.value,
        "data-invert": invert.value,
        "data-swipe-out": swipeOut.value,
        "data-expanded": Boolean(_ctx.expanded || _ctx.expandByDefault && mounted.value),
        style: normalizeStyle({
          ...unref(assignInlineVars)({
            [unref(indexVar)]: `${_ctx.index}`,
            [unref(toastsBeforeVar)]: `${_ctx.index}`,
            [unref(zIndexVar)]: `${_ctx.toasts.length - _ctx.index}`,
            [unref(offsetVar)]: `${removed.value ? offsetBeforeRemove.value : offset.value}px`,
            [unref(initialHeightVar)]: _ctx.expandByDefault ? "auto" : `${initialHeight.value}px`,
            [unref(swipeAmountVar)]: `${swipeAmount.value}`
          }),
          ..._ctx.rawCSS,
          ...unref(toast).rawCSS
        }),
        onPointerdown: handleOnPointerDown,
        onPointerup: handleOnPointerUp,
        onPointermove: handleOnPointerMove,
        class: normalizeClass(unref(clx)(unref(toast), props.class ?? props.className, unref(toast).className))
      }, [
        _ctx.closeButton ? (openBlock(), createElementBlock("button", {
          key: 0,
          "aria-label": "Close toast",
          "data-disabled": disabled.value,
          "data-close-button": true,
          onClick: _cache[0] || (_cache[0] = ($event) => handleCloseButtonClick()),
          class: normalizeClass(unref(toastCloseButton))
        }, _hoisted_4, 10, _hoisted_2)) : createCommentVNode("", true),
        toastType.value || unref(toast).icon || unref(toast).promise ? (openBlock(), createElementBlock("div", {
          key: 1,
          "data-icon": "",
          class: normalizeClass(unref(toastIcon))
        }, [
          unref(toast).promise && toastType.value === "loading" ? (openBlock(), createElementBlock("div", {
            key: 0,
            class: normalizeClass(unref(toastSpinner))
          }, [
            createVNode(unref(_sfc_main$1), { name: "loaderFill" })
          ], 2)) : createCommentVNode("", true),
          unref(toast).icon ? (openBlock(), createElementBlock(Fragment, { key: 1 }, [
            createTextVNode(toDisplayString(unref(toast).icon), 1)
          ], 64)) : createCommentVNode("", true),
          !unref(toast).icon && statusIconName.value ? (openBlock(), createBlock(unref(_sfc_main$1), {
            key: 2,
            name: statusIconName.value
          }, null, 8, ["name"])) : createCommentVNode("", true)
        ], 2)) : createCommentVNode("", true),
        createElementVNode("div", {
          "data-content": "",
          class: normalizeClass(unref(toastContent))
        }, [
          createElementVNode("div", {
            "data-title": "",
            class: normalizeClass(unref(toastTitle))
          }, toDisplayString(unref(toast).title), 3),
          unref(toast).description ? (openBlock(), createElementBlock("div", {
            key: 0,
            "data-description": "",
            class: normalizeClass(unref(clx)(unref(toastDescription), _ctx.descriptionClassName))
          }, toDisplayString(unref(toast).description), 3)) : createCommentVNode("", true)
        ], 2),
        unref(toast).cancel ? (openBlock(), createElementBlock("button", {
          key: 2,
          "data-button": true,
          "data-cancel": true,
          onClick: _cache[1] || (_cache[1] = ($event) => handleCancelButtonClick()),
          class: normalizeClass(unref(toastCancelButton))
        }, toDisplayString(unref(toast).cancel.label), 3)) : createCommentVNode("", true),
        unref(toast).action ? (openBlock(), createElementBlock("button", {
          key: 3,
          "data-button": "",
          onClick: _cache[2] || (_cache[2] = ($event) => handleActionButtonClick($event)),
          class: normalizeClass(unref(toastButton))
        }, toDisplayString(unref(toast).action.label), 3)) : createCommentVNode("", true)
      ], 46, _hoisted_1);
    };
  }
});

export { _sfc_main as default };
