'use strict';

Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: 'Module' } });

const vue = require('vue');
const dom = require('@floating-ui/dom');
const box_vue_vue_type_script_setup_true_lang = require('../box/box.vue.cjs');
const shared_css = require('../shared/shared.css.cjs');

const _sfc_main = /* @__PURE__ */ vue.defineComponent({
  __name: "tooltip",
  props: {
    title: { default: void 0 },
    placement: { default: "top" },
    offset: { default: void 0 },
    surroundPadding: { default: void 0 },
    children: { default: void 0 },
    className: {},
    class: {},
    classList: {},
    forwardedRef: {}
  },
  setup(__props) {
    const props = __props;
    const isMounted = vue.ref(false);
    const isShown = vue.ref(false);
    const anchorRef = vue.ref();
    const tooltipRef = vue.ref();
    const arrowRef = vue.ref();
    vue.onMounted(() => {
      isMounted.value = true;
    });
    vue.watch(
      () => [
        isMounted.value,
        isShown.value,
        props.placement,
        props.offset,
        props.surroundPadding
      ],
      () => {
        if (!isMounted.value) {
          return;
        }
        compute();
      },
      { immediate: true }
    );
    function setTooltip(shouldShow) {
      if (shouldShow) {
        isShown.value = true;
        compute();
      } else {
        isShown.value = false;
      }
    }
    function getLatestRefs() {
      return {
        arrow: arrowRef.value,
        anchor: anchorRef.value,
        tooltip: tooltipRef.value
      };
    }
    function compute() {
      const latestRefs = getLatestRefs();
      if (latestRefs.anchor == null || latestRefs.tooltip == null) {
        return;
      }
      dom.computePosition(latestRefs.anchor, latestRefs.tooltip, {
        placement: props.placement ?? "top-start",
        middleware: [
          dom.inline(),
          dom.offset(props.offset ?? 12),
          dom.flip(),
          dom.shift({
            padding: props.surroundPadding ?? 4
          }),
          dom.arrow({
            element: arrowRef.value
          }),
          dom.size({
            apply({ availableWidth, availableHeight, elements }) {
              Object.assign(elements.floating.style, {
                maxWidth: `${availableWidth}px`,
                maxHeight: `${availableHeight}px`
              });
            }
          })
        ]
      }).then((result) => {
        const x = result.x;
        const y = result.y;
        const placement = result.placement;
        const arrowData = result.middlewareData.arrow;
        const { x: arrowX, y: arrowY } = arrowData;
        Object.assign(latestRefs.tooltip.style, {
          left: `${x}px`,
          top: `${y}px`
        });
        const staticSide = {
          top: "bottom",
          right: "left",
          bottom: "top",
          left: "right"
        }[placement.split("-")[0]];
        const endSide = {
          start: "start",
          end: "end"
        }[placement.split("-")[1]];
        const deltaX = endSide === "start" ? -12 : endSide === "end" ? 12 : 0;
        Object.assign(latestRefs.arrow.style, {
          left: arrowX != null ? `${arrowX + deltaX}px` : "",
          top: arrowY != null ? `${arrowY}px` : "",
          right: "",
          bottom: "",
          [staticSide]: "-4px"
        });
      });
    }
    return (_ctx, _cache) => {
      return vue.openBlock(), vue.createElementBlock(vue.Fragment, null, [
        vue.createVNode(vue.unref(box_vue_vue_type_script_setup_true_lang.default), { attributes: {
          "data-part-id": "tooltip-container"
        } }, {
          default: vue.withCtx(() => [
            vue.createVNode(vue.unref(box_vue_vue_type_script_setup_true_lang.default), {
              display: "flex",
              alignItems: "center",
              cursor: "help",
              boxRef: anchorRef.value,
              attributes: {
                tabIndex: "0",
                onMouseEnter: () => setTooltip(true),
                onMouseLeave: () => setTooltip(false),
                onFocus: () => setTooltip(true),
                onBlur: () => setTooltip(false)
              }
            }, {
              default: vue.withCtx(() => [
                vue.renderSlot(_ctx.$slots, "default")
              ]),
              _: 3
            }, 8, ["boxRef", "attributes"])
          ]),
          _: 3
        }),
        vue.createVNode(vue.unref(box_vue_vue_type_script_setup_true_lang.default), {
          px: "$5",
          py: "$3",
          backgroundColor: "$text",
          borderRadius: "$md",
          position: "absolute",
          width: "max-content",
          zIndex: "1",
          attributes: {
            role: "tooltip"
          },
          display: isShown.value ? "block" : "none",
          boxRef: tooltipRef.value,
          rawCSS: {
            left: "0",
            top: "0"
          },
          class: vue.normalizeClass(vue.unref(shared_css.standardTransitionProperties))
        }, {
          default: vue.withCtx(() => [
            vue.createTextVNode(vue.toDisplayString(_ctx.title), 1),
            vue.createVNode(vue.unref(box_vue_vue_type_script_setup_true_lang.default), {
              position: "absolute",
              transform: "rotate(45deg)",
              width: "$5",
              height: "$5",
              backgroundColor: "$text",
              boxRef: arrowRef.value,
              attributes: {
                "data-part-id": "tooltip-arrow"
              }
            }, null, 8, ["boxRef"])
          ]),
          _: 1
        }, 8, ["display", "boxRef", "class"])
      ], 64);
    };
  }
});

exports.default = _sfc_main;
